SystemOrganization addCategory: 'OrbeonBPMXpdl'!

!XPDLTransitionNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/29/2015 13:13'!
createBpmDefinitionOn: aBpmProcessDefinition

	^(BpmTransitionDefinition process: aBpmProcessDefinition)
		name: self getName;
		setCondition: startCondition;
		yourself! !

Object subclass: #BpmProcessGenerator
	instanceVariableNames: 'xpdlPackages processes participants processesMapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrbeonBPMXpdl'!

BpmProcessGenerator subclass: #BpmBizagiProcessGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrbeonBPMXpdl'!

!BpmProcessGenerator class methodsFor: 'other' stamp: 'BrunoBB 11/22/2015 16:25'!
exampleSmalltalksRegistration
	| generator |

	generator := self on: (XPDLReader2 exampleSmalltalksRegistration) xpdlPackages.

	generator 
		mapProcess: 'registration' toApplication: (OrbeonApplication applicationNamed: 'st');
		generateAllProcess.

	^generator! !

!BpmProcessGenerator class methodsFor: 'other' stamp: 'BrunoBB 11/28/2015 12:28'!
exampleTestCaseFixIssueReportArchitect
	| generator |

	generator := self on: (XPDLReader2 exampleMultipleProcesses) xpdlPackages.

	generator 
		mapProcess: 'test-case-process' toApplication: (OrbeonApplication applicationNamed: 'ue');
		mapProcess: 'fix-issue' toApplication: (OrbeonApplication applicationNamed: 'ue');
		mapProcess: 'architect-process' toApplication: (OrbeonApplication applicationNamed: 'ue');
		generateAllProcess.

	^generator! !

!BpmProcessGenerator class methodsFor: 'other' stamp: 'BrunoBB 10/31/2015 15:57'!
on: collectionOfXPDLPackages

	^super new initialize
		xpdlPackages: collectionOfXPDLPackages;
		yourself! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 10/31/2015 16:41'!
addParticipantsOf: aXPDLPackageNode to: aBpmProcessDefinition

	^aBpmProcessDefinition addAllParticipants: participants! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 10/30/2015 12:55'!
connectSubProcessTaskWithSubProcess
	"The receiver connect all sub process tasks <BpmSubProcessTaskDefinition> with the related process definition <BpmProcessDefinition>"
	| subProcessTasks |

	subProcessTasks := OrderedCollection new.

	processes do: [:procDefinition | 
		procDefinition getSubProcessTasks do: [:subProcTaskDefinition | 
			subProcTaskDefinition subProcessDefinition: (processes detect: [:each | each sheetName = subProcTaskDefinition name] ifNone:  [nil]).
		].
	].! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 11/01/2015 08:19'!
createParticipantDefinitionsOf: aXPDLPackageNode
	"The receiver create the GLOBAL participants <BpmParticipantDefinition> for all processes to be imported"

	aXPDLPackageNode participants do: [:participantNode |
		participants add: participantNode createBpmDefinition]! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 11/02/2015 14:34'!
generateAllProcess
	"The receiver creates all Processes from <xpdlPackages>.
	1- Create the GLOBAL Participants for all processes.
	2- Create one process <BpmProcessDefinition> per each pool <XPDLPoolNode> in all packages <XPDLPackageNode>
	3- Connect all sub processes node <BpmSubProcessTaskDefinition> with the concrete processes definitions <BpmProcessDefinition>"

	xpdlPackages isEmpty ifTrue: [^self].

	self createParticipantDefinitionsOf: xpdlPackages asArray first. "participants are GLOBAL to all sheets in a Bizagi model with subprocesses"

	xpdlPackages do: [:eachPackage | 
		eachPackage pools do: [:each | |  newProcess |
			newProcess := self generateProcessBasedOn: each.
			newProcess xmlString: eachPackage prettyPrinted. "set the XML string of the process"
			processes at: newProcess processName put: newProcess.
			self addParticipantsOf: eachPackage to: newProcess.
		].
	].

	self connectSubProcessTaskWithSubProcess. "connect sub process task  artifacts with process definitions"

	^processes! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 11/29/2015 17:16'!
generateProcessBasedOn: aXPDLPool
	"The receiver creates BPM process definition <BpmProcessDefinition> based on the XPDL pool <aXPDLPool>"
	| processDefinition roles application |

	processDefinition := (self getProcessDefinitionClassFrom: aXPDLPool) newNamed: aXPDLPool getName.
	processDefinition sheetName: aXPDLPool getSheetName.
	application := processesMapping at: processDefinition processName ifAbsent: [self throwMappingExceptionFor: processDefinition processName].
	processDefinition orbeonApplication: application.

	roles := Dictionary new.
	aXPDLPool lanes keysDo: [:laneName | roles at: laneName put: (BpmFrontOfficeRole createNewOrGetExisting: laneName in: application app)].

	"it create the corresponding Bpm definition for each Xpdl artifact <xpdlActivity> in the pool <aXPDLPool>"
	aXPDLPool activities do: [:xpdlActivity | | newArtifact lanesNames |
		newArtifact := xpdlActivity createBpmDefinitionOn: processDefinition.
		newArtifact addToParentProcess.
		lanesNames := (xpdlActivity getLanesFrom: aXPDLPool) collect: [:each | each getName].
		lanesNames do: [:laneName | (roles at: laneName ifAbsent: [nil]) ifNotNil: [:rol | newArtifact grantRole: rol]]
		].

	aXPDLPool transitions collect: [:xpdlTransition | | newTransition |
		newTransition := (xpdlTransition createBpmDefinitionOn: processDefinition) addToParentProcess.
		newTransition setStartArtifact: (xpdlTransition startNode getArtifactFrom: processDefinition).
		newTransition setEndArtifact: (xpdlTransition endNode getArtifactFrom: processDefinition).
	].

	^processDefinition! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 11/29/2015 17:37'!
getProcessDefinitionClassFrom: aXPDLPool
	"Answer the correct process definition for xpdl element <aXPDLPool>"

	^aXPDLPool hasParentProcess
		ifFalse: [BpmTopProcessDefinition]
		ifTrue: [BpmSubProcessDefinition]! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 10/29/2015 13:53'!
getProcessDefinitionNamed: aString ifNone: aBlock

	^processes at: aString ifAbsent: aBlock! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 11/22/2015 14:28'!
initialize

	super initialize.

	processes := Dictionary new.
	participants := OrderedCollection new.
	processesMapping := Dictionary new. "each key is a process name and each value is a valid OrbeonApplication name"! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 12/09/2015 11:17'!
mapProcess: procNamed toApplication: anOrbeonApplication
	"The receiver map the BpmProcessDefinition named <procNamed> with the <OrbeonApplication> named <appName>"

	^processesMapping at: procNamed put: anOrbeonApplication! !

!BpmProcessGenerator methodsFor: 'Accessing' stamp: 'BrunoBB 10/31/2015 16:29'!
participants
	^participants! !

!BpmProcessGenerator methodsFor: 'Updating' stamp: 'BrunoBB 10/31/2015 16:29'!
participants: newValue
	participants := newValue! !

!BpmProcessGenerator methodsFor: 'Accessing' stamp: 'BrunoBB 10/26/2015 17:48'!
processes
	^processes! !

!BpmProcessGenerator methodsFor: 'Updating' stamp: 'BrunoBB 10/26/2015 17:48'!
processes: newValue
	processes := newValue! !

!BpmProcessGenerator methodsFor: 'other' stamp: 'BrunoBB 11/22/2015 14:46'!
throwMappingExceptionFor: aString

	^ProcessAppMappingException signal: 'Failed to map [', aString, '] with an OrbeonApplication'! !

!BpmProcessGenerator methodsFor: 'Accessing' stamp: 'BrunoBB 10/31/2015 15:54'!
xpdlPackages
	^xpdlPackages! !

!BpmProcessGenerator methodsFor: 'Updating' stamp: 'BrunoBB 10/31/2015 15:54'!
xpdlPackages: newValue
	xpdlPackages := newValue! !

!XPDLStartNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/27/2015 16:15'!
createBpmDefinitionOn: aBpmProcessDefinition

	^(BpmStartEventDefinition process: aBpmProcessDefinition)
		name: self getName;
		yourself! !

!XPDLStartNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/28/2015 12:18'!
getArtifactFrom: aBpmProcessDefinition

	^aBpmProcessDefinition startEvent! !

!XPDLIntermediateNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/27/2015 16:15'!
createBpmDefinitionOn: aBpmProcessDefinition

	^(BpmIntermediateEventDefinition process: aBpmProcessDefinition)
		name: self getName;
		yourself! !

!XPDLIntermediateNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/28/2015 12:16'!
getArtifactFrom: aBpmProcessDefinition

	^aBpmProcessDefinition getIntermediateEventNamed: self getName ifNone: [nil]! !

!XPDLTaskNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/28/2015 12:16'!
getArtifactFrom: aBpmProcessDefinition

	^aBpmProcessDefinition getTaskNamed: self getName ifNone: [nil]! !

!XPDLTaskNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 11/01/2015 14:17'!
getBpmClass

	^BpmTaskDefinition! !

!XPDLGatewayNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/30/2015 14:14'!
createBpmDefinitionOn: aBpmProcessDefinition

	^(self getGatewayClass process: aBpmProcessDefinition)
		name: self getName;
		yourself! !

!XPDLGatewayNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/28/2015 12:15'!
getArtifactFrom: aBpmProcessDefinition

	^aBpmProcessDefinition getGatewayNamed: self getName ifNone: [nil]! !

!XPDLGatewayNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/30/2015 14:15'!
getGatewayClass

	(type = 'Parallel') ifTrue: [^BpmParallelGatewayDefinition].

	(type = 'Inclusive') ifTrue: [^BpmInclusiveGatewayDefinition].

	(type = 'Complex') ifTrue: [^BpmComplexGatewayDefinition].

	^BpmExclusiveGatewayDefinition! !

Exception subclass: #OrbeonXpdlException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrbeonBPMXpdl'!

OrbeonXpdlException subclass: #ProcessAppMappingException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OrbeonBPMXpdl'!

!XPDLParticipantNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/31/2015 16:24'!
createBpmDefinition

	^(self getType = 'ROLE')
		ifTrue: [BpmRoleParticipantDefinition new name: self getName] 
		ifFalse:  [BpmResourceParticipantDefinition new name: self getName]! !

!XPDLSubProcessNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/28/2015 12:17'!
getArtifactFrom: aBpmProcessDefinition

	^aBpmProcessDefinition getSubProcessTaskNamed: self getName ifNone: [nil]! !

!XPDLSubProcessNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 11/01/2015 14:17'!
getBpmClass

	^BpmSubProcessTaskDefinition! !

!XPDLActivityNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 11/01/2015 14:47'!
createBpmDefinitionOn: aBpmProcessDefinition
	| loop activityDefinition |

	activityDefinition := (self getBpmClass process: aBpmProcessDefinition)
		name: self getName;
		yourself.

	loop := self getLoopTypeClass for: activityDefinition.
	loop startQuantity: self getStartQuantity;
		completionQuantity: self getCompletionQuantity;
		setLoop: loopCondition.

	^activityDefinition loop: loop! !

!XPDLActivityNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 11/01/2015 14:29'!
getBpmClass

	^self subclassResponsibility! !

!XPDLActivityNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 11/01/2015 14:29'!
getLoopTypeClass

	(loopType = 'Standard') ifTrue: [^BpmStandartTaskLoop].

	(loopType = 'MultiInstance') ifTrue: [^BpmMultiInstanceTaskLoop].

	(loopType = 'None') ifTrue: [^BpmNoneTaskLoop].

	^BpmNoneTaskLoop! !

!XPDLEndNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/27/2015 16:14'!
createBpmDefinitionOn: aBpmProcessDefinition

	^(BpmEndEventDefinition process: aBpmProcessDefinition)
		name: self getName;
		yourself! !

!XPDLEndNode methodsFor: '*orbeonBPMXpdl' stamp: 'BrunoBB 10/28/2015 12:16'!
getArtifactFrom: aBpmProcessDefinition

	^aBpmProcessDefinition getEndEventNamed: self getName ifNone: [nil]! !
